# The spawner is based on PRP's https://gitlab.nrp-nautilus.io/vpeddu/jupyterlab-west.git
# This file is managed by [youngsu.kim|dung.vu]@csusb.edu
from kubespawner import KubeSpawner


class MySpawner(KubeSpawner):
    with open('/etc/jupyterhub/custom/my_spawner.html', 'r') as f:
        profile_form_template = f.readlines()
    profile_form_template = "".join(profile_form_template)

    def options_from_form(self, formdata):
        if not self.profile_list or not hasattr(self, '_profile_list'):
            return formdata
        selected_profile = int(formdata.get('profile', [0])[0])
        options = self._profile_list[selected_profile]
        self.log.debug("Applying KubeSpawner override for profile '%s'", options['display_name'])
        kubespawner_override = options.get('kubespawner_override', {})
        for k, v in kubespawner_override.items():
            if callable(v):
                v = v(self)
                self.log.debug(".. overriding KubeSpawner value %s=%s (callable result)", k, v)
            else:
                self.log.debug(".. overriding KubeSpawner value %s=%s", k, v)
            setattr(self, k, v)

        gpus = int(formdata.get('gpus', [0])[0])
        
        setattr(self, "extra_resource_limits", {"nvidia.com/gpu": gpus})
        setattr(self, "mem_guarantee", formdata.get('ram', [0])[0]+"G")
        setattr(self, "cpu_guarantee", float(formdata.get('cores', [0])[0]))
        setattr(self, "mem_limit", formdata.get('ram', [0])[0]+"G")
        setattr(self, "cpu_limit", float(formdata.get('cores', [0])[0]))

        nodeSelectorTermsExpressions = [
            {
            'key': 'topology.kubernetes.io/region',
            'operator': 'In',
            'values': ["us-west","us-central"]
            }
        ]

        if formdata.get('gputype', [0])[0]:
            nodeSelectorTermsExpressions.append({
                'key': 'nvidia.com/gpu.product',
                'operator': 'In',
                'values': formdata.get('gputype', [0])
            })

        tolerationsExpression = [{
            'key': 'nautilus.io/csu-tide',
            'operator': 'Exists',
            'effect': 'NoSchedule',
            },
            {
            'key': 'nautilus.io/sdsu-fix',
            'operator': 'Exists',
            'effect': 'NoSchedule',
            }
        ]


        setattr(self, 'extra_pod_config', {
                    'securityContext': {
                        'fsGroupChangePolicy': 'OnRootMismatch',
                        'fsGroup': 100
                    },
                    'affinity': {
                        'nodeAffinity': {
                            'requiredDuringSchedulingIgnoredDuringExecution': {
                                'nodeSelectorTerms': [{
                                    'matchExpressions': nodeSelectorTermsExpressions,
                                }],
                            },
                        },
                    },
                    'tolerations': tolerationsExpression,
                    })

     

        setattr(self, 'startTimeout', 1200)
        
        self.volume_mounts = [
            {
                'name': 'volume-{username}',
                'mountPath': '/home/jovyan',
            },
            {
                'name': 'csusb-hpc-share-drive',
                'mountPath': '/home/jovyan/shared',
            },
            {
                'name': 'shm-volume',
                'mountPath': '/dev/shm',
            },
            {
                'name': 'scratch',
                'mountPath': '/home/jovyan/scratch-drive',
            },
            ]
        self.volumes = [
            {
                'name': 'volume-{username}',
                'persistentVolumeClaim': {
                    'claimName': 'claim-{username}'
                    }
            },
            {
                'name': 'csusb-hpc-share-drive',
                'persistentVolumeClaim': {
                    'claimName': 'csusb-hpc-share-sweed'
                    }
            },
            {
                'name': 'shm-volume',
                'emptyDir': {
                    'medium': 'Memory'
                    }
            },
            {
                'name': 'scratch',
                'emptyDir': {}
            },
            ]

        return options

    profile_list = [
        {
            'display_name': 'Stack Minimal',
        },
        {
            'display_name': 'Stack Datascience',
            'kubespawner_override': {
                'image_spec': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/datascience:v1.3.1',
                'imagePullPolicy': 'IfNotPresent',
            }
        },
        {
            'display_name': 'Stack Desktop Apps - VS Code',
            'kubespawner_override': {
                'image': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/5g-desktop:v1.3.1',
                'imagePullPolicy': 'IfNotPresent',
                'lifecycle_hooks': {
                    'postStart': {
                        'exec': {
                            'command': [
                                'sh',
                                '-c',
                                'mkdir -p /home/jovyan/Desktop && cp /home/jovyan/shared/.software/desktopInit.desktop /home/jovyan/Desktop/ || true'
                            ]
                        }
                    }
                }
            }
        },  
        {
            'display_name': 'Stack Desktop Apps - Pgadmin4',
            'kubespawner_override': {
#                 OS 22.04 pdadmin only
                'image': 'gitlab-registry.nrp-nautilus.io/csusb-prp/csusb-jupyter-stacks/desktop-plus:v1.3.1',
                'imagePullPolicy': 'IfNotPresent',
                'lifecycle_hooks': {
                    'postStart': {
                        'exec': {
                            'command': [
                                'sh',
                                '-c',
                                'mkdir -p /home/jovyan/Desktop && cp /home/jovyan/shared/.software/desktopInit.desktop /home/jovyan/Desktop/ || true'
                            ]
                        }
                    }
                }

            }
        },
        {
            'display_name': "Stack Desktop Apps - Blender",
            'kubespawner_override': {
# blender is installed in image                
                'image': 'gitlab-registry.nrp-nautilus.io/csusb-prp/csusb-jupyter-stacks/desktop-blender:v1.3.1',
# Blender is installed in Shared
#                'image': 'gitlab-registry.nrp-nautilus.io/csusb-prp/csusb-jupyter-stacks/desktop-plus:6e0795b6',
#                'image': 'gitlab-registry.nrp-nautilus.io/csusb-prp/csusb-jupyter-stacks/desktop-plus:a0544c38',
                'imagePullPolicy': 'IfNotPresent',
                'lifecycle_hooks': {
                    'postStart': {
                        'exec': {
                            'command': [
                                'sh',
                                '-c',
                                'mkdir -p /home/jovyan/Desktop && cp /home/jovyan/shared/.software/desktopInit.desktop /home/jovyan/Desktop/ || true'
                            ]
                        }
                    }
                }
            }
        }
,
        {
            'display_name': 'Stack PySpark',
            'kubespawner_override': {
                'image_spec': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/pyspark:v1.3.1',
                'imagePullPolicy': 'IfNotPresent',
            }
        },

        {
            'display_name': 'Stack PyTorch2',
            'kubespawner_override': {
                'image': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/prp-pytorch2:v1.0',
                'imagePullPolicy': 'IfNotPresent',
            }
        },       
        {
            'display_name': 'Stack R-Studio',
            'kubespawner_override': {
                'image': 'gitlab-registry.nrp-nautilus.io/csusb-prp/csusb-jupyter-stacks/r-studio-notebook:v1.3.1',                
                'image_spec': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/r-studio:v1.3.1',
 #               'image_spec': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/r-studio:v1.3.1',
                'imagePullPolicy': 'IfNotPresent',
            }
        },
        {
            'display_name': 'Stack R-Studio (Becerra)',
            'kubespawner_override': {
                'image_spec': 'gitlab-registry.nrp-nautilus.io/youngsu_kim/csusb-jupyter-stack/stack-rstudio-becerra-class',
                'imagePullPolicy': 'IfNotPresent',
            }
        },
        {
            'display_name': 'Stack SageMath',
            'kubespawner_override': {
                'image_spec': 'gitlab-registry.nrp-nautilus.io/youngsu_kim/csusb-jupyter-stack/stack-sagemath',
                # 'imagePullPolicy': 'IfNotPresent',
            }
        },
        {
             'display_name': 'prp - tensorflow 2.91',
             'kubespawner_override': {
                 'image': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/prp:v1.0',
                 'imagePullPolicy': 'IfNotPresent',
             }
        },
        {
            'display_name': 'Stack Tensorflow + PRP Packages',
            'kubespawner_override': {
                'image_spec': 'gitlab-registry.nrp-nautilus.io/prp/jupyter-stack/prp:v1.3.1',
                'imagePullPolicy': 'IfNotPresent',
            }
        },
        {
            'display_name': 'Stack Zeus-Cling',
            'kubespawner_override': {
                'image_spec': 'gitlab-registry.nrp-nautilus.io/youngsu_kim/csusb-jupyter-stack/stack-xeus-cling',
                'imagePullPolicy': 'IfNotPresent',
            }
        },
    ]

c.JupyterHub.spawner_class = MySpawner
